/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

/**
   Draw a scene with five instances of the "square
   within a square" model from the previous example.

   This example shows a "hack" that draws five copies
   of the decorated square by using "multiple exposures"
   into the framebuffer of a single model object that
   gets moved around in between the "exposures".

   This works in an "offline" rendering, but it will not
   work for an interactive renderer (like a game) that
   needs all five models on the screen at the same time.
*/
public class Example2
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();

      final Model decoratedSquare = new Model("Decorated Square");
      decoratedSquare.addVertex(new Vertex(-1, -1, -3),
                                new Vertex(-1,  0, -3),
                                new Vertex(-1,  1, -3),
                                new Vertex( 0,  1, -3),
                                new Vertex( 1,  1, -3),
                                new Vertex( 1,  0, -3),
                                new Vertex( 1, -1, -3),
                                new Vertex( 0, -1, -3));
      decoratedSquare.addLineSegment(new LineSegment(0, 2),
                                     new LineSegment(2, 4),
                                     new LineSegment(4, 6),
                                     new LineSegment(6, 0),
                                     new LineSegment(1, 3),
                                     new LineSegment(3, 5),
                                     new LineSegment(5, 7),
                                     new LineSegment(7, 1));
      scene.addModel(decoratedSquare);

      final FrameBuffer fb = new FrameBuffer(800, 800);

      // render once
      Pipeline.render(scene, fb);

      // move the model
      for (int i = 0; i < decoratedSquare.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare.vertexList.get(i);
         decoratedSquare.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                      v.y + 1.5,   // move up
                                                      v.z - 2.5)); // move back
      }

      // render the model a second time into the same framebuffer
      Pipeline.render(scene, fb);

      // move the model again
      for (int i = 0; i < decoratedSquare.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare.vertexList.get(i);
         decoratedSquare.vertexList.set(i, new Vertex(v.x - 3.0,   // move left
                                                      v.y,
                                                      v.z));
      }

      // render a third time into the same framebuffer
      Pipeline.render(scene, fb);

      // move the model again
      for (int i = 0; i < decoratedSquare.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare.vertexList.get(i);
         decoratedSquare.vertexList.set(i, new Vertex(v.x,
                                                      v.y - 3.0, // move down
                                                      v.z));
      }

      // render a fourth time into the framebuffer
      Pipeline.render(scene, fb);

      // move the model again
      for (int i = 0; i < decoratedSquare.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare.vertexList.get(i);
         decoratedSquare.vertexList.set(i, new Vertex(v.x + 3.0, // move right
                                                      v.y,
                                                      v.z));
      }

      // render a fifth time into the same framebuffer
      Pipeline.render(scene, fb);

      fb.dumpFB2File("example2.ppm");
   }
}
