/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

/**
   Draw a scene with five instances of the "square
   within a square" model from the previous example.

   This time, we want all five instances of the model in
   the scene at the same time. So we need five instances of
   the model object. We end up with a lot of repeated code.
*/
public class Example3
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();

      final Model decoratedSquare1 = new Model("Decorated Square");
      decoratedSquare1.addVertex(new Vertex(-1, -1, -3),
                                 new Vertex(-1,  0, -3),
                                 new Vertex(-1,  1, -3),
                                 new Vertex( 0,  1, -3),
                                 new Vertex( 1,  1, -3),
                                 new Vertex( 1,  0, -3),
                                 new Vertex( 1, -1, -3),
                                 new Vertex( 0, -1, -3));
      decoratedSquare1.addLineSegment(new LineSegment(0, 2),
                                      new LineSegment(2, 4),
                                      new LineSegment(4, 6),
                                      new LineSegment(6, 0),
                                      new LineSegment(1, 3),
                                      new LineSegment(3, 5),
                                      new LineSegment(5, 7),
                                      new LineSegment(7, 1));
      scene.addModel(decoratedSquare1); // add this model to the scene

      final Model decoratedSquare2 = new Model("Decorated Square 2");
      decoratedSquare2.addVertex(new Vertex(-1, -1, -3),
                                new Vertex(-1,  0, -3),
                                new Vertex(-1,  1, -3),
                                new Vertex( 0,  1, -3),
                                new Vertex( 1,  1, -3),
                                new Vertex( 1,  0, -3),
                                new Vertex( 1, -1, -3),
                                new Vertex( 0, -1, -3));
      decoratedSquare2.addLineSegment(new LineSegment(0, 2),
                                     new LineSegment(2, 4),
                                     new LineSegment(4, 6),
                                     new LineSegment(6, 0),
                                     new LineSegment(1, 3),
                                     new LineSegment(3, 5),
                                     new LineSegment(5, 7),
                                     new LineSegment(7, 1));
      scene.addModel(decoratedSquare2); // add this model to the scene
      for (int i = 0; i < decoratedSquare2.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare2.vertexList.get(i);
         decoratedSquare2.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                       v.y + 1.5,   // move up
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare3 = new Model("Decorated Square 3");
      decoratedSquare3.addVertex(new Vertex(-1, -1, -3),
                                 new Vertex(-1,  0, -3),
                                 new Vertex(-1,  1, -3),
                                 new Vertex( 0,  1, -3),
                                 new Vertex( 1,  1, -3),
                                 new Vertex( 1,  0, -3),
                                 new Vertex( 1, -1, -3),
                                 new Vertex( 0, -1, -3));
      decoratedSquare3.addLineSegment(new LineSegment(0, 2),
                                      new LineSegment(2, 4),
                                      new LineSegment(4, 6),
                                      new LineSegment(6, 0),
                                      new LineSegment(1, 3),
                                      new LineSegment(3, 5),
                                      new LineSegment(5, 7),
                                      new LineSegment(7, 1));
      scene.addModel(decoratedSquare3); // add this model to the scene
      for (int i = 0; i < decoratedSquare3.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare3.vertexList.get(i);
         decoratedSquare3.vertexList.set(i, new Vertex(v.x - 1.5,   // move left
                                                       v.y + 1.5,   // move up
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare4 = new Model("Decorated Square 4");
      decoratedSquare4.addVertex(new Vertex(-1, -1, -3),
                                 new Vertex(-1,  0, -3),
                                 new Vertex(-1,  1, -3),
                                 new Vertex( 0,  1, -3),
                                 new Vertex( 1,  1, -3),
                                 new Vertex( 1,  0, -3),
                                 new Vertex( 1, -1, -3),
                                 new Vertex( 0, -1, -3));
      decoratedSquare4.addLineSegment(new LineSegment(0, 2),
                                      new LineSegment(2, 4),
                                      new LineSegment(4, 6),
                                      new LineSegment(6, 0),
                                      new LineSegment(1, 3),
                                      new LineSegment(3, 5),
                                      new LineSegment(5, 7),
                                      new LineSegment(7, 1));
      scene.addModel(decoratedSquare4); // add this model to the scene
      for (int i = 0; i < decoratedSquare4.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare4.vertexList.get(i);
         decoratedSquare4.vertexList.set(i, new Vertex(v.x - 1.5,   // move left
                                                       v.y - 1.5,   // move down
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare5 = new Model("Decorated Square 5");
      decoratedSquare5.addVertex(new Vertex(-1, -1, -3),
                                 new Vertex(-1,  0, -3),
                                 new Vertex(-1,  1, -3),
                                 new Vertex( 0,  1, -3),
                                 new Vertex( 1,  1, -3),
                                 new Vertex( 1,  0, -3),
                                 new Vertex( 1, -1, -3),
                                 new Vertex( 0, -1, -3));
      decoratedSquare5.addLineSegment(new LineSegment(0, 2),
                                      new LineSegment(2, 4),
                                      new LineSegment(4, 6),
                                      new LineSegment(6, 0),
                                      new LineSegment(1, 3),
                                      new LineSegment(3, 5),
                                      new LineSegment(5, 7),
                                      new LineSegment(7, 1));
      scene.addModel(decoratedSquare5); // add this model to the scene
      for (int i = 0; i < decoratedSquare5.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare5.vertexList.get(i);
         decoratedSquare5.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                       v.y - 1.5,   // move down
                                                       v.z - 2.5)); // move back
      }

      final FrameBuffer fb = new FrameBuffer(800, 800);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("example3.ppm");
   }
}
