/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

/**
   We can get rid of a lot of the repeated code in the last
   example by abstracting the "decorated square" into a
   subclass of Model. The we can call its constructor five
   times and get five instances that we can place in the
   scene. The DecoratedSquare subclass is at the end of
   this file as a private class.

   http://people.scs.carleton.ca/~lanthier/teaching/COMP1406/Notes/COMP1406_Ch4_ClassHierarchiesAndInheritance.pdf
   http://people.scs.carleton.ca/~lanthier/teaching/COMP1406/notes.html
*/
public class Example4
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();

      final Model decoratedSquare1 = new DecoratedSquare_E4("Decorated Square 1");
      scene.addModel(decoratedSquare1);

      final Model decoratedSquare2 = new DecoratedSquare_E4("Decorated Square 2");
      scene.addModel(decoratedSquare2);
      for (int i = 0; i < decoratedSquare2.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare2.vertexList.get(i);
         decoratedSquare2.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                       v.y + 1.5,   // move up
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare3 = new DecoratedSquare_E4("Decorated Square 3");
      scene.addModel(decoratedSquare3);
      for (int i = 0; i < decoratedSquare3.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare3.vertexList.get(i);
         decoratedSquare3.vertexList.set(i, new Vertex(v.x - 1.5,   // move left
                                                       v.y + 1.5,   // move up
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare4 = new DecoratedSquare_E4("Decorated Square 4");
      scene.addModel(decoratedSquare4);
      for (int i = 0; i < decoratedSquare4.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare4.vertexList.get(i);
         decoratedSquare4.vertexList.set(i, new Vertex(v.x - 1.5,   // move left
                                                       v.y - 1.5,   // move down
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare5 = new DecoratedSquare_E4("Decorated Square 5");
      scene.addModel(decoratedSquare5);
      for (int i = 0; i < decoratedSquare5.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare5.vertexList.get(i);
         decoratedSquare5.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                       v.y - 1.5,   // move down
                                                       v.z - 2.5)); // move back
      }

      final FrameBuffer fb = new FrameBuffer(800, 800);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("example4.ppm");
   }
}


/**
   Notice how what was the repeated code in the last example
   becomes the code in the constructor of this subclass of Model.
*/
class DecoratedSquare_E4 extends Model
{
   public DecoratedSquare_E4(final String name)
   {
      super(name);

      this.addVertex(new Vertex(-1, -1, -3),
                     new Vertex(-1,  0, -3),
                     new Vertex(-1,  1, -3),
                     new Vertex( 0,  1, -3),
                     new Vertex( 1,  1, -3),
                     new Vertex( 1,  0, -3),
                     new Vertex( 1, -1, -3),
                     new Vertex( 0, -1, -3));
      this.addLineSegment(new LineSegment(0, 2),
                          new LineSegment(2, 4),
                          new LineSegment(4, 6),
                          new LineSegment(6, 0),
                          new LineSegment(1, 3),
                          new LineSegment(3, 5),
                          new LineSegment(5, 7),
                          new LineSegment(7, 1));
   }
}
