/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

/**
   In this example we move the subclass of Model into
   its own file. That makes the code in the subclass
   more reusable (since a private class in a file
   cannot be used outside of that file).
*/
public class Example5
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();

      final Model decoratedSquare1 = new DecoratedSquare("Decorated Square 1");
      scene.addModel(decoratedSquare1);

      final Model decoratedSquare2 = new DecoratedSquare("Decorated Square 2");
      scene.addModel(decoratedSquare2);
      for (int i = 0; i < decoratedSquare2.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare2.vertexList.get(i);
         decoratedSquare2.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                       v.y + 1.5,   // move up
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare3 = new DecoratedSquare("Decorated Square 3");
      scene.addModel(decoratedSquare3);
      for (int i = 0; i < decoratedSquare3.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare3.vertexList.get(i);
         decoratedSquare3.vertexList.set(i, new Vertex(v.x - 1.5,   // move left
                                                       v.y + 1.5,   // move up
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare4 = new DecoratedSquare("Decorated Square 4");
      scene.addModel(decoratedSquare4);
      for (int i = 0; i < decoratedSquare4.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare4.vertexList.get(i);
         decoratedSquare4.vertexList.set(i, new Vertex(v.x - 1.5,   // move left
                                                       v.y - 1.5,   // move down
                                                       v.z - 2.5)); // move back
      }

      final Model decoratedSquare5 = new DecoratedSquare("Decorated Square 5");
      scene.addModel(decoratedSquare5);
      for (int i = 0; i < decoratedSquare5.vertexList.size(); ++i)
      {
         final Vertex v = decoratedSquare5.vertexList.get(i);
         decoratedSquare5.vertexList.set(i, new Vertex(v.x + 1.5,   // move right
                                                       v.y - 1.5,   // move down
                                                       v.z - 2.5)); // move back
      }

      final FrameBuffer fb = new FrameBuffer(800, 800);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("example5.ppm");
   }
}
